from memberaudit.app_settings import MEMBERAUDIT_APP_NAME
from memberaudit.models import CharacterUpdateStatus

from django.template.loader import render_to_string
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

from allianceauth.authentication.models import CharacterOwnership

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)


def dashboard_memberaudit_check(request):
    unregistered = CharacterOwnership.objects.filter(
        user=request.user, character__memberaudit_character__isnull=True
    )

    registred = CharacterOwnership.objects.filter(user=request.user).values_list(
        "character__memberaudit_character", flat=True
    )

    issues = CharacterUpdateStatus.objects.filter(character__in=registred, is_success=0)

    chars = {}

    if unregistered or issues:
        for char in unregistered:
            title = _("Character Registration Issue")
            msg = f"<span class='text-danger'><i class='fas fa-times-circle' data-bs-tooltip='aa-memberaudit-dashboard' title='{title}'></i></span>"
            chars[char.character.character_id] = {
                "id": char.character.character_id,
                "name": char.character.character_name,
                "issues": _(f"Character is not registered in {MEMBERAUDIT_APP_NAME}."),
                "icon": format_html(msg),
            }

    if issues:
        for issue in issues:
            if issue.character.eve_character.character_id not in chars:
                title = _("Character Update Issue")
                msg = f"<span class='text-warning'><i class='fas fa-triangle-exclamation' data-bs-tooltip='aa-memberaudit-dashboard' title='{title}'></i></span>"
                chars[issue.character.eve_character.character_id] = {
                    "id": issue.character.eve_character.character_id,
                    "name": issue.character.eve_character.character_name,
                    "issues": _(
                        "Please re-register this character, as there was an issue with the last update."
                    ),
                    "icon": format_html(msg),
                }

    context = {
        "chars": chars if chars else None,
        "memberaudit_app_name": MEMBERAUDIT_APP_NAME,
    }
    return render_to_string(
        "madashboard/dashboard.html", context=context, request=request
    )



from unittest.mock import MagicMock

from django.http import HttpResponse
from django.test import RequestFactory, TestCase

from app_utils.testing import create_user_from_evecharacter

from madashboard.auth_hooks import MemberCheckDashboardHook, register_membercheck_hook
from madashboard.tests.testdata.load_allianceauth import load_allianceauth
from madashboard.tests.testdata.load_memberaudit import load_memberaudit


class TestAuthHooks(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()
        load_memberaudit()
        cls.factory = RequestFactory()
        cls.user_without_permission, cls.character_ownership = (
            create_user_from_evecharacter(character_id=1002)
        )
        cls.user_with_ma_permission, cls.character_ownership = (
            create_user_from_evecharacter(
                character_id=1001,
                permissions=["memberaudit.basic_access"],
            )
        )

    def test_render_returns_empty_string_for_user_without_permission(self):
        # given
        request = self.factory.get("/")
        request.user = self.user_without_permission
        rendered_item = MemberCheckDashboardHook()

        # when
        response = rendered_item.render(request)
        # Convert SafeString to HttpResponse for testing
        response = HttpResponse(response)
        # then
        self.assertEqual(response.status_code, 200)
        self.assertNotIn(
            '<div id="memberaudit-check-dashboard-widget" class="col-12 mb-3">',
            response.content.decode("utf-8"),
        )

    def test_render_returns_widget_for_user_with_permission(self):
        # given
        request = self.factory.get("/")
        request.user = self.user_with_ma_permission
        rendered_item = MemberCheckDashboardHook()

        # when
        response = rendered_item.render(request)
        # Convert SafeString to HttpResponse for testing
        response = HttpResponse(response)
        # then
        self.assertEqual(response.status_code, 200)
        self.assertIn(
            '<div id="memberaudit-check-dashboard-widget" class="col-12 mb-3">',
            response.content.decode("utf-8"),
        )

    def test_register_membercheck_hook(self):
        # given
        hooks = register_membercheck_hook()

        # then
        self.assertIsInstance(hooks, MemberCheckDashboardHook)



"""Hook into Alliance Auth"""

# Django
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth import hooks

from .views import dashboard_memberaudit_check


class MemberCheckDashboardHook(hooks.DashboardItemHook):
    def __init__(self):
        super().__init__(view_function=dashboard_memberaudit_check, order=5)


@hooks.register("dashboard_hook")
def register_membercheck_hook():
    return MemberCheckDashboardHook()



"""App Configuration"""

# Django
from django.apps import AppConfig

from madashboard import __version__


class MADashbaordConfig(AppConfig):
    """App Config"""

    default_auto_field = "django.db.models.AutoField"
    name = "madashboard"
    label = "madashboard"
    verbose_name = f"MemberAudit Dashboard v{__version__}"
